typedef struct 
	{
	LocalID		appLocalID;					// TSut:temp
	FieldPtr	fldP;						// BlkS:temp
	UInt16		chr;						// BlkS:temp
	Boolean		capsLock;					// FTgl:temp
	Boolean		numLock;					// FTgl:temp
	UInt16		tempShift;					// FTgl:temp
	Boolean		autoShifted;				// FTgl:temp
	UInt32 		OnF:1;						// TSut
	UInt32 		OffF:1;						// TSut
	UInt32 		AutoEnbleAfterResetF:1;		// TSut
	UInt32 		BlockAutoShiftingF:1;		// BlkS
	UInt32		MenuHwrKeyControlF:1;		// WbMn
	UInt32		ToolbarHwrKeyControlF:1;	// WbMn
	UInt32		OpenHwrKeyControlF:1;		// WbMn
	UInt32		UpDownPushDetected:1;		// WbMn:temp
	UInt32		prevCmdBtnIndex:8;			// WbMn:temp
	UInt32		numButtons:8;				// WbMn:temp
	UInt32		keyControlHandled:1;		// WbMn:temp
	UInt32		FEPModeControlF:1;			// FTgl
	UInt32 		AddNichiEiBtnF:1;			// Ftgl
	UInt32 		AddHiraKataBtnF:1;			// FTgl
	UInt32 		AddHenkanBtnF:1;			// FTgl
	UInt32 		AddBtnBackLitONF:1;			// FTgl
	UInt32 		UnUsed:2;					// UnUsed
	UInt8		BtnInsPosition;				// WbMn
	} TSuitePreferenceType;

#define IsOn		1
#define IsOff		0
#define Activate	1
#define Deactivate	0

#define appFileCreator			'TSut'
#define appVersionNum			0x01
#define appPrefID				0x01
#define appPrefVersionNum		0x01

#define FEPBtnDbName			"T-Suite.FEPBtn.PalmLo"

#define ftr_appStatus			0x00	//Ftr

#define BlkSPriority			0x00	// sysNotifyEventDequeuedEvent
#define WbMnPriority			0x01	// sysNotifyEventDequeuedEvent
#define WbMmPriority			0x02	// sysNotifyEventDequeuedEvent
#define WbMtPriority			0x03	// sysNotifyMenuCmdBarOpenEvent
										// customNotify 'TSut'
										// sysNotifyEventDequeuedEvent
#define FTglPriority			0x04	// sysNotifyMenuCmdBarOpenEvent

#define Btn_NichiEiBitmapFamily		14997
#define Btn_HiraKataBitmapFamily	14998
#define Btn_HenkanBitmapFamily		14999

#define Swap16(n) (n)

#pragma mark ----------

Err BlkSEventDequedHandle (SysNotifyParamType *notifyParamsP);

Err WbMnUpDownKeyEventPreHandle (SysNotifyParamType *notifyParamsP);

Err WbMmMenuKeyDownHandle (SysNotifyParamType *notifyParamsP);

Err WbMtCmdBarOpenHandle (SysNotifyParamType *notifyParamsP);
Err WbMtCmdBarOpenedHandle (SysNotifyParamType *notifyParamsP);
Err WbMtCmdBarKeyDownHandle (SysNotifyParamType *notifyParamsP);
void WbMtCmdBarDrawRect (void);										// used in FTgl
void WbMtCmdBarKeyDownResDraw (Char **nameP, UInt8 numButtons);		// used in FTgl

Err FTglCmdBarOpenHandle (SysNotifyParamType *notifyParamsP);
Err FTglCmdBarOpenedHandle (SysNotifyParamType *notifyParamsP);
Err FTglCmdBarKeyDownHandle (SysNotifyParamType *notifyParamsP);
